import numpy as np
from .metric import Metric

class Accuracy(Metric):
    """ Accuracy.

    Args:
        per_pixel (boolean, optional): whether to normalize the error with number of pixels. Default: True.
    """

    def __init__(self):
        super().__init__()
        self.reset()

    def reset(self):
        self.correct = 0
        self.total = 0

    def add(self, predicted, target):
        """
        Args:
            predicted (numpy.ndarray) : prediction from the model
            target (numpy.ndarray) : target output value
        """
        predicted = np.argmax(predicted, axis=1)
        self.total += target.shape[0]
        self.correct += (predicted == target).sum().item()
        
    def value(self):
        """
        Returns:
            Accuracy as a percentage
        """
        return 100 * self.correct/max(self.total, 1)



import numpy as np
from .metric import Metric


class ConfusionMatrix(Metric):
    """Constructs a confusion matrix for a multi-class classification problems.

    Does not support multi-label, multi-class problems.

    Keyword arguments:
    - num_classes (int): number of classes in the classification problem.
    - normalized (boolean, optional): Determines whether or not the confusion
    matrix is normalized or not. Default: False.

    Modified from: https://github.com/pytorch/tnt/blob/master/torchnet/meter/confusionmeter.py
    """

    def __init__(self, num_classes, normalized=False):
        super().__init__()

        self.conf = np.ndarray((num_classes, num_classes), dtype=np.int64)
        self.normalized = normalized
        self.num_classes = num_classes
        self.reset()

    def reset(self):
        self.conf.fill(0)

    def add(self, predicted, target):
        """Computes the confusion matrix

        The shape of the confusion matrix is K x K, where K is the number
        of classes.

        Keyword arguments:
        - predicted (numpy.ndarray): Can be an N x K array of
        predicted scores obtained from the model for N examples and K classes,
        or an N array of integer values between 0 and K-1.
        - target (numpy.ndarray): Can be an N x K array of
        ground-truth classes for N examples and K classes, or an N array
        of integer values between 0 and K-1.

        """
        assert predicted.shape[0] == target.shape[0], \
            'number of targets and predicted outputs do not match'

        if np.ndim(predicted) != 1:
            assert predicted.shape[1] == self.num_classes, \
                'number of predictions does not match size of confusion matrix'
            predicted = np.argmax(predicted, 1)
        else:
            assert (predicted.max() < self.num_classes) and (predicted.min() >= 0), \
                'predicted values are not between 0 and k-1'

        if np.ndim(target) != 1:
            assert target.shape[1] == self.num_classes, \
                'Onehot target does not match size of confusion matrix'
            assert (target >= 0).all() and (target <= 1).all(), \
                'in one-hot encoding, target values should be 0 or 1'
            assert (target.sum(1) == 1).all(), \
                'multi-label setting is not supported'
            target = np.argmax(target, 1)
        else:
            assert (target.max() < self.num_classes) and (target.min() >= 0), \
                'target values are not between 0 and k-1'

        # hack for bincounting 2 arrays together
        x = predicted + self.num_classes * target
        bincount_2d = np.bincount(
            x.astype(np.int64), minlength=self.num_classes**2)
        assert bincount_2d.size == self.num_classes**2
        conf = bincount_2d.reshape((self.num_classes, self.num_classes))

        self.conf += conf

    def value(self):
        """
        Returns:
            Confustion matrix of K rows and K columns, where rows corresponds
            to ground-truth targets and columns corresponds to predicted
            targets.
        """
        if self.normalized:
            conf = self.conf.astype(np.float32)
            return conf / conf.sum(1).clip(min=1e-12)[:, None]
        else:
            return self.conf



class Metric(object):
    """Base class for all metrics.

    From: https://github.com/pytorch/tnt/blob/master/torchnet/meter/meter.py
    """
    def reset(self):
        pass

    def add(self):
        pass

    def value(self):
        pass


import os, sys
import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

# "setup.py publish" shortcut.
if sys.argv[-1] == 'publish':
    os.system('python setup.py sdist')
    os.system('twine upload dist/*')
    os.system('rm -rf dist pytorch_trainer_tensorboard.egg-info')
    sys.exit()


setuptools.setup(
    name='pytorch-trainer-tensorboard',  
    version='0.4.8',
    license='MIT',
    author="Vamshi Chowdary Madala",
    author_email="vamchowdary72@gmail.com",
    description="A simple trainer class for pytorch with tensorboard support",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/vamshichowdary/pytorch-trainer",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=[]                     # Install other dependencies if any
)



import numpy as np
from .metric import Metric


class L1Error(Metric):
    """L1 error.

    Args:
        per_pixel (boolean, optional): whether to normalize the error with number of pixels. Default: True.
        ignore_value (None or tensor) : Pixels with value in target to ignore (useful for eg if some target pixels contain NaNs and we do not 
        want these pixels to contribute to the total error)
    """

    def __init__(self, per_pixel=True, ignore_value=None):
        super().__init__()
        
        self.reset()
        self.per_pixel = per_pixel
        self.ignore_value = ignore_value

    def reset(self):
        self.n = 0
        self.l1_sum = 0.

    def add(self, predicted, target):
        """Computes the total L1 error (Absolute error)
        Args:
            predicted (numpy.ndarray) : prediction from the model
            target (numpy.ndarray) : target output value
        """
        if self.ignore_value is not None:
            mask = np.not_equal(target, self.ignore_value)
            self.l1_sum += np.abs(mask*(predicted-target)).sum()
            self.n += mask.sum()
        else:
            self.l1_sum += np.abs(predicted-target).sum()
            self.n += target.size
        
    def value(self):
        """
        Returns:
            L1 error (or normalized per pixel error (MAE))
        """
        if self.per_pixel:
            return self.l1_sum / max(self.n, 1)
        else:
            return self.l1_sum
